@@ -16,6 +16,8 @@
 
 """Python implementation of _search_key functions, etc."""
 
+from __future__ import absolute_import
+
 import zlib
 import struct
 
@@ -157,3 +159,11 @@ def _deserialise_internal_node(bytes, key, search_key_func=None):
     result._node_width = len(prefix)
     result._search_prefix = common_prefix
     return result
+
+
+def _bytes_to_text_key(bytes):
+    """Take a CHKInventory value string and return a (file_id, rev_id) tuple"""
+    sections = bytes.split('\n')
+    kind, file_id = sections[0].split(': ')
+    return (intern(file_id), intern(sections[3]))
+
