@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """A version of inspect that includes what 'copy' needs.
 
@@ -25,6 +25,8 @@ This module is designed to use tricky hacks in import rules, to avoid this
 overhead.
 """
 
+from __future__ import absolute_import
+
 
 ####
 # These are the only 2 functions that 'copy' needs from 'inspect'
