@@ -14,61 +14,27 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from __future__ import absolute_import
+
+from bzrlib.errors import (
+    BinaryFiles,
+    MalformedHunkHeader,
+    MalformedLine,
+    MalformedPatchHeader,
+    PatchConflict,
+    PatchSyntax,
+    )
+
 import re
 
 
 binary_files_re = 'Binary files (.*) and (.*) differ\n'
 
 
-class BinaryFiles(Exception):
-
-    def __init__(self, orig_name, mod_name):
-        self.orig_name = orig_name
-        self.mod_name = mod_name
-        Exception.__init__(self, 'Binary files section encountered.')
-
-
-class PatchSyntax(Exception):
-    def __init__(self, msg):
-        Exception.__init__(self, msg)
-
-
-class MalformedPatchHeader(PatchSyntax):
-    def __init__(self, desc, line):
-        self.desc = desc
-        self.line = line
-        msg = "Malformed patch header.  %s\n%r" % (self.desc, self.line)
-        PatchSyntax.__init__(self, msg)
-
-
-class MalformedHunkHeader(PatchSyntax):
-    def __init__(self, desc, line):
-        self.desc = desc
-        self.line = line
-        msg = "Malformed hunk header.  %s\n%r" % (self.desc, self.line)
-        PatchSyntax.__init__(self, msg)
-
-
-class MalformedLine(PatchSyntax):
-    def __init__(self, desc, line):
-        self.desc = desc
-        self.line = line
-        msg = "Malformed line.  %s\n%s" % (self.desc, self.line)
-        PatchSyntax.__init__(self, msg)
-
-
-class PatchConflict(Exception):
-    def __init__(self, line_no, orig_line, patch_line):
-        orig = orig_line.rstrip('\n')
-        patch = str(patch_line).rstrip('\n')
-        msg = 'Text contents mismatch at line %d.  Original has "%s",'\
-            ' but patch says it should be "%s"' % (line_no, orig, patch)
-        Exception.__init__(self, msg)
-
-
 def get_patch_names(iter_lines):
+    line = iter_lines.next()
     try:
-        line = iter_lines.next()
         match = re.match(binary_files_re, line)
         if match is not None:
             raise BinaryFiles(match.group(1), match.group(2))
@@ -250,7 +216,13 @@ class Hunk:
         return shift
 
 
-def iter_hunks(iter_lines):
+def iter_hunks(iter_lines, allow_dirty=False):
+    '''
+    :arg iter_lines: iterable of lines to parse for hunks
+    :kwarg allow_dirty: If True, when we encounter something that is not
+        a hunk header when we're looking for one, assume the rest of the lines
+        are not part of the patch (comments or other junk).  Default False
+    '''
     hunk = None
     for line in iter_lines:
         if line == "\n":
@@ -260,7 +232,15 @@ def iter_hunks(iter_lines):
             continue
         if hunk is not None:
             yield hunk
-        hunk = hunk_from_header(line)
+        try:
+            hunk = hunk_from_header(line)
+        except MalformedHunkHeader:
+            if allow_dirty:
+                # If the line isn't a hunk header, then we've reached the end
+                # of this patch and there's "junk" at the end.  Ignore the
+                # rest of this patch.
+                return
+            raise
         orig_size = 0
         mod_size = 0
         while orig_size < hunk.orig_range or mod_size < hunk.mod_range:
@@ -338,8 +318,12 @@ class Patch(BinaryPatch):
                 if isinstance(line, ContextLine):
                     pos += 1
 
-
-def parse_patch(iter_lines):
+def parse_patch(iter_lines, allow_dirty=False):
+    '''
+    :arg iter_lines: iterable of lines to parse
+    :kwarg allow_dirty: If True, allow the patch to have trailing junk.
+        Default False
+    '''
     iter_lines = iter_lines_handle_nl(iter_lines)
     try:
         (orig_name, mod_name) = get_patch_names(iter_lines)
@@ -347,17 +331,44 @@ def parse_patch(iter_lines):
         return BinaryPatch(e.orig_name, e.mod_name)
     else:
         patch = Patch(orig_name, mod_name)
-        for hunk in iter_hunks(iter_lines):
+        for hunk in iter_hunks(iter_lines, allow_dirty):
             patch.hunks.append(hunk)
         return patch
 
 
-def iter_file_patch(iter_lines):
+def iter_file_patch(iter_lines, allow_dirty=False, keep_dirty=False):
+    '''
+    :arg iter_lines: iterable of lines to parse for patches
+    :kwarg allow_dirty: If True, allow comments and other non-patch text
+        before the first patch.  Note that the algorithm here can only find
+        such text before any patches have been found.  Comments after the
+        first patch are stripped away in iter_hunks() if it is also passed
+        allow_dirty=True.  Default False.
+    '''
+    ### FIXME: Docstring is not quite true.  We allow certain comments no
+    # matter what, If they startwith '===', '***', or '#' Someone should
+    # reexamine this logic and decide if we should include those in
+    # allow_dirty or restrict those to only being before the patch is found
+    # (as allow_dirty does).
     regex = re.compile(binary_files_re)
     saved_lines = []
+    dirty_head = []
     orig_range = 0
+    beginning = True
+
     for line in iter_lines:
-        if line.startswith('=== ') or line.startswith('*** '):
+        if line.startswith('=== '):
+            if len(saved_lines) > 0:
+                if keep_dirty and len(dirty_head) > 0:
+                    yield {'saved_lines': saved_lines,
+                           'dirty_head': dirty_head}
+                    dirty_head = []
+                else:
+                    yield saved_lines
+                saved_lines = []
+            dirty_head.append(line)
+            continue
+        if line.startswith('*** '):
             continue
         if line.startswith('#'):
             continue
@@ -365,15 +376,29 @@ def iter_file_patch(iter_lines):
             if line.startswith('-') or line.startswith(' '):
                 orig_range -= 1
         elif line.startswith('--- ') or regex.match(line):
-            if len(saved_lines) > 0:
-                yield saved_lines
+            if allow_dirty and beginning:
+                # Patches can have "junk" at the beginning
+                # Stripping junk from the end of patches is handled when we
+                # parse the patch
+                beginning = False
+            elif len(saved_lines) > 0:
+                if keep_dirty and len(dirty_head) > 0:
+                    yield {'saved_lines': saved_lines,
+                           'dirty_head': dirty_head}
+                    dirty_head = []
+                else:
+                    yield saved_lines
             saved_lines = []
         elif line.startswith('@@'):
             hunk = hunk_from_header(line)
             orig_range = hunk.orig_range
         saved_lines.append(line)
     if len(saved_lines) > 0:
-        yield saved_lines
+        if keep_dirty and len(dirty_head) > 0:
+            yield {'saved_lines': saved_lines,
+                   'dirty_head': dirty_head}
+        else:
+            yield saved_lines
 
 
 def iter_lines_handle_nl(iter_lines):
@@ -397,8 +422,24 @@ def iter_lines_handle_nl(iter_lines):
         yield last_line
 
 
-def parse_patches(iter_lines):
-    return [parse_patch(f.__iter__()) for f in iter_file_patch(iter_lines)]
+def parse_patches(iter_lines, allow_dirty=False, keep_dirty=False):
+    '''
+    :arg iter_lines: iterable of lines to parse for patches
+    :kwarg allow_dirty: If True, allow text that's not part of the patch at
+        selected places.  This includes comments before and after a patch
+        for instance.  Default False.
+    :kwarg keep_dirty: If True, returns a dict of patches with dirty headers.
+        Default False.
+    '''
+    patches = []
+    for patch_lines in iter_file_patch(iter_lines, allow_dirty, keep_dirty):
+        if 'dirty_head' in patch_lines:
+            patches.append({'patch': parse_patch(
+                patch_lines['saved_lines'], allow_dirty),
+                            'dirty_head': patch_lines['dirty_head']})
+        else:
+            patches.append(parse_patch(patch_lines, allow_dirty))
+    return patches
 
 
 def difference_index(atext, btext):
