@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 """Tests for the 'check' CLI command."""
 
 from bzrlib.tests import ChrootedTestCase
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests import TestCaseWithTransport
 
 
-class TestCheck(ExternalBase):
+class TestCheck(TestCaseWithTransport):
 
     def test_check_no_tree(self):
         self.make_branch('.')
@@ -34,23 +34,22 @@ class TestCheck(ExternalBase):
         tree = self.make_branch_and_tree('.')
         tree.commit('hallelujah')
         out, err = self.run_bzr('check')
-        self.assertContainsRe(err, r"^Checking working tree at '.*'\.\n"
-                                   r"Checking repository at '.*'\.\n"
-                                   r"checked repository.*\n"
+        self.assertContainsRe(err, r"Checking working tree at '.*'\.\n")
+        self.assertContainsRe(err, r"Checking repository at '.*'\.\n")
+        # the root directory may be in the texts for rich root formats
+        self.assertContainsRe(err, r"checked repository.*\n"
                                    r"     1 revisions\n"
-                                   r"     0 file-ids\n"
-                                   r"     0 unique file texts\n"
-                                   r"     0 repeated file texts\n"
-                                   r"     0 unreferenced text versions\n"
-                                   r"Checking branch at '.*'\.\n"
-                                   r"checked branch.*\n$")
+                                   r"     [01] file-ids\n"
+                                   )
+        self.assertContainsRe(err, r"Checking branch at '.*'\.\n")
+        self.assertContainsRe(err, r"checked branch.*")
 
     def test_check_branch(self):
         tree = self.make_branch_and_tree('.')
         tree.commit('foo')
         out, err = self.run_bzr('check --branch')
         self.assertContainsRe(err, r"^Checking branch at '.*'\.\n"
-                                   r"checked branch.*\n$")
+                                   r"checked branch.*")
 
     def test_check_repository(self):
         tree = self.make_branch_and_tree('.')
@@ -59,10 +58,8 @@ class TestCheck(ExternalBase):
         self.assertContainsRe(err, r"^Checking repository at '.*'\.\n"
                                    r"checked repository.*\n"
                                    r"     1 revisions\n"
-                                   r"     0 file-ids\n"
-                                   r"     0 unique file texts\n"
-                                   r"     0 repeated file texts\n"
-                                   r"     0 unreferenced text versions$")
+                                   r"     [01] file-ids\n"
+                                   )
 
     def test_check_tree(self):
         tree = self.make_branch_and_tree('.')
@@ -76,7 +73,7 @@ class TestCheck(ExternalBase):
         out, err = self.run_bzr('check --tree --branch')
         self.assertContainsRe(err, r"^Checking working tree at '.*'\.\n"
                                    r"Checking branch at '.*'\.\n"
-                                   r"checked branch.*\n$")
+                                   r"checked branch.*")
 
     def test_check_missing_tree(self):
         branch = self.make_branch('.')
@@ -87,9 +84,9 @@ class TestCheck(ExternalBase):
         branch = self.make_branch('.')
         out, err = self.run_bzr('check --tree --branch')
         self.assertContainsRe(err,
-            r"^No working tree found at specified location\.\n"
             r"Checking branch at '.*'\.\n"
-            r"checked branch.*\n$")
+            r"No working tree found at specified location\.\n"
+            r"checked branch.*")
 
     def test_check_missing_branch_in_shared_repo(self):
         self.make_repository('shared', shared=True)
