@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Black-box tests for 'bzr inventory'."""
 
@@ -24,7 +24,7 @@ from bzrlib.tests import TestCaseWithTransport
 class TestInventory(TestCaseWithTransport):
 
     def setUp(self):
-        TestCaseWithTransport.setUp(self)
+        super(TestInventory, self).setUp()
 
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/', 'b/c'])
