@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2011 Canonical Ltd
+# Copyright (C) 2006-2012, 2016 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@ import re
 from bzrlib import (
     branch,
     bzrdir,
+    controldir,
     errors,
     osutils,
     tests,
@@ -32,12 +33,12 @@ from bzrlib import (
     )
 from bzrlib.repofmt import knitrepo
 from bzrlib.tests import (
-    blackbox,
     http_server,
     scenarios,
+    script,
     test_foreign,
-    test_server,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.transport import memory
 
 
@@ -56,6 +57,26 @@ class TestPush(tests.TestCaseWithTransport):
                            ['push', public_url],
                            working_dir='source')
 
+    def test_push_suggests_parent_alias(self):
+        """Push suggests using :parent if there is a known parent branch."""
+        tree_a = self.make_branch_and_tree('a')
+        tree_a.commit('this is a commit')
+        tree_b = self.make_branch_and_tree('b')
+
+        # If there is no parent location set, :parent isn't mentioned.
+        out = self.run_bzr('push', working_dir='a', retcode=3)
+        self.assertEqual(out,
+                ('','bzr: ERROR: No push location known or specified.\n'))
+
+        # If there is a parent location set, the error suggests :parent.
+        tree_a.branch.set_parent(tree_b.branch.base)
+        out = self.run_bzr('push', working_dir='a', retcode=3)
+        self.assertEqual(out,
+            ('','bzr: ERROR: No push location known or specified. '
+                'To push to the parent branch '
+                '(at %s), use \'bzr push :parent\'.\n' %
+                urlutils.unescape_for_display(tree_b.branch.base, 'utf-8')))
+
     def test_push_remember(self):
         """Push changes from one branch to another and test push location."""
         transport = self.get_transport()
@@ -79,42 +100,47 @@ class TestPush(tests.TestCaseWithTransport):
 
         # test push for failure without push location set
         out = self.run_bzr('push', working_dir='branch_a', retcode=3)
-        self.assertEquals(out,
+        self.assertEqual(out,
                 ('','bzr: ERROR: No push location known or specified.\n'))
 
         # test not remembered if cannot actually push
         self.run_bzr('push path/which/doesnt/exist',
                      working_dir='branch_a', retcode=3)
         out = self.run_bzr('push', working_dir='branch_a', retcode=3)
-        self.assertEquals(
+        self.assertEqual(
                 ('', 'bzr: ERROR: No push location known or specified.\n'),
                 out)
 
         # test implicit --remember when no push location set, push fails
         out = self.run_bzr('push ../branch_b',
                            working_dir='branch_a', retcode=3)
-        self.assertEquals(out,
+        self.assertEqual(out,
                 ('','bzr: ERROR: These branches have diverged.  '
                  'See "bzr help diverged-branches" for more information.\n'))
-        self.assertEquals(osutils.abspath(branch_a.get_push_location()),
+        # Refresh the branch as 'push' modified it
+        branch_a = branch_a.bzrdir.open_branch()
+        self.assertEqual(osutils.abspath(branch_a.get_push_location()),
                           osutils.abspath(branch_b.bzrdir.root_transport.base))
 
         # test implicit --remember after resolving previous failure
         uncommit.uncommit(branch=branch_b, tree=tree_b)
         transport.delete('branch_b/c')
         out, err = self.run_bzr('push', working_dir='branch_a')
+        # Refresh the branch as 'push' modified it
+        branch_a = branch_a.bzrdir.open_branch()
         path = branch_a.get_push_location()
-        self.assertEquals(out,
-                          'Using saved push location: %s\n'
-                          % urlutils.local_path_from_url(path))
         self.assertEqual(err,
+                         'Using saved push location: %s\n'
                          'All changes applied successfully.\n'
-                         'Pushed up to revision 2.\n')
+                         'Pushed up to revision 2.\n'
+                         % urlutils.local_path_from_url(path))
         self.assertEqual(path,
                          branch_b.bzrdir.root_transport.base)
         # test explicit --remember
         self.run_bzr('push ../branch_c --remember', working_dir='branch_a')
-        self.assertEquals(branch_a.get_push_location(),
+        # Refresh the branch as 'push' modified it
+        branch_a = branch_a.bzrdir.open_branch()
+        self.assertEqual(branch_a.get_push_location(),
                           branch_c.bzrdir.root_transport.base)
 
     def test_push_without_tree(self):
@@ -135,7 +161,7 @@ class TestPush(tests.TestCaseWithTransport):
         out, err = self.run_bzr('push --no-tree -d push-from push-to')
         self.assertEqual('', out)
         self.assertEqual('Created new branch.\n', err)
-        self.failIfExists('push-to/file')
+        self.assertPathDoesNotExist('push-to/file')
 
     def test_push_new_branch_revision_count(self):
         # bzr push of a branch with revisions to a new location
@@ -149,6 +175,23 @@ class TestPush(tests.TestCaseWithTransport):
         self.assertEqual('', out)
         self.assertEqual('Created new branch.\n', err)
 
+    def test_push_quiet(self):
+        # test that using -q makes output quiet
+        t = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file'])
+        t.add('file')
+        t.commit('commit 1')
+        self.run_bzr('push -d tree pushed-to')
+        # Refresh the branch as 'push' modified it and get the push location
+        push_loc = t.branch.bzrdir.open_branch().get_push_location()
+        out, err = self.run_bzr('push', working_dir="tree")
+        self.assertEqual('Using saved push location: %s\n'
+                         'No new revisions or tags to push.\n' %
+                         urlutils.local_path_from_url(push_loc), err)
+        out, err = self.run_bzr('push -q', working_dir="tree")
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+
     def test_push_only_pushes_history(self):
         # Knit branches should only push the history for the current revision.
         format = bzrdir.BzrDirMetaFormat1()
@@ -158,7 +201,7 @@ class TestPush(tests.TestCaseWithTransport):
 
         def make_shared_tree(path):
             shared_repo.bzrdir.root_transport.mkdir(path)
-            shared_repo.bzrdir.create_branch_convenience('repo/' + path)
+            controldir.ControlDir.create_branch_convenience('repo/' + path)
             return workingtree.WorkingTree.open('repo/' + path)
         tree_a = make_shared_tree('a')
         self.build_tree(['repo/a/file'])
@@ -199,10 +242,26 @@ class TestPush(tests.TestCaseWithTransport):
         t.commit(allow_pointless=True,
                 message='first commit')
         self.run_bzr('push -d from to-one')
-        self.failUnlessExists('to-one')
+        self.assertPathExists('to-one')
         self.run_bzr('push -d %s %s'
             % tuple(map(urlutils.local_path_to_url, ['from', 'to-two'])))
-        self.failUnlessExists('to-two')
+        self.assertPathExists('to-two')
+
+    def test_push_repository_no_branch_doesnt_fetch_all_revs(self):
+        # See https://bugs.launchpad.net/bzr/+bug/465517
+        target_repo = self.make_repository('target')
+        source = self.make_branch_builder('source')
+        source.start_series()
+        source.build_snapshot('A', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        source.build_snapshot('B', ['A'], [])
+        source.build_snapshot('C', ['A'], [])
+        source.finish_series()
+        self.run_bzr('push target -d source')
+        self.addCleanup(target_repo.lock_read().unlock)
+        # We should have pushed 'C', but not 'B', since it isn't in the
+        # ancestry
+        self.assertEqual([('A',), ('C',)], sorted(target_repo.revisions.keys()))
 
     def test_push_smart_non_stacked_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -216,6 +275,8 @@ class TestPush(tests.TestCaseWithTransport):
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(9, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
 
     def test_push_smart_stacked_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -231,7 +292,9 @@ class TestPush(tests.TestCaseWithTransport):
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(13, self.hpss_calls)
+        self.assertLength(15, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
         remote = branch.Branch.open('public')
         self.assertEndsWith(remote.get_stacked_on_url(), '/parent')
 
@@ -248,6 +311,8 @@ class TestPush(tests.TestCaseWithTransport):
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(11, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
 
     def test_push_smart_incremental_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -264,6 +329,8 @@ class TestPush(tests.TestCaseWithTransport):
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(11, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
 
     def test_push_smart_with_default_stacking_url_path_segment(self):
         # If the default stacked-on location is a path element then branches
@@ -308,7 +375,7 @@ class TestPush(tests.TestCaseWithTransport):
                      working_dir='tree')
         new_tree = workingtree.WorkingTree.open('new/tree')
         self.assertEqual(tree.last_revision(), new_tree.last_revision())
-        self.failUnlessExists('new/tree/a')
+        self.assertPathExists('new/tree/a')
 
     def test_push_use_existing(self):
         """'bzr push --use-existing-dir' can push into an existing dir.
@@ -329,7 +396,7 @@ class TestPush(tests.TestCaseWithTransport):
         new_tree = workingtree.WorkingTree.open('target')
         self.assertEqual(tree.last_revision(), new_tree.last_revision())
         # The push should have created target/a
-        self.failUnlessExists('target/a')
+        self.assertPathExists('target/a')
 
     def test_push_use_existing_into_empty_bzrdir(self):
         """'bzr push --use-existing-dir' into a dir with an empty .bzr dir
@@ -385,6 +452,8 @@ class TestPush(tests.TestCaseWithTransport):
         self.assertTrue(repo_to.has_revision('from-1'))
         self.assertFalse(repo_to.has_revision('from-2'))
         self.assertEqual(tree_to.branch.last_revision_info()[1], 'from-1')
+        self.assertFalse(
+            tree_to.changes_from(tree_to.basis_tree()).has_changed())
 
         self.run_bzr_error(
             ['bzr: ERROR: bzr push --revision '
@@ -446,7 +515,8 @@ class TestPush(tests.TestCaseWithTransport):
         trunk_public = self.make_branch('public_trunk', format='1.9')
         trunk_public.pull(trunk_tree.branch)
         trunk_public_url = self.get_readonly_url('public_trunk')
-        trunk_tree.branch.set_public_branch(trunk_public_url)
+        br = trunk_tree.branch
+        br.set_public_branch(trunk_public_url)
         # now we do a stacked push, which should determine the public location
         # for us.
         out, err = self.run_bzr(['push', '--stacked',
@@ -545,6 +615,22 @@ class TestPush(tests.TestCaseWithTransport):
         self.assertEqual('', out)
         self.assertEqual('Created new branch.\n', err)
 
+    def test_overwrite_tags(self):
+        """--overwrite-tags only overwrites tags, not revisions."""
+        from_tree = self.make_branch_and_tree('from')
+        from_tree.branch.tags.set_tag("mytag", "somerevid")
+        to_tree = self.make_branch_and_tree('to')
+        to_tree.branch.tags.set_tag("mytag", "anotherrevid")
+        revid1 = to_tree.commit('my commit')
+        out = self.run_bzr(['push', '-d', 'from', 'to'])
+        self.assertEqual(out,
+            ('Conflicting tags:\n    mytag\n', 'No new revisions to push.\n'))
+        out = self.run_bzr(['push', '-d', 'from', '--overwrite-tags', 'to'])
+        self.assertEqual(out, ('', '1 tag updated.\n'))
+        self.assertEqual(to_tree.branch.tags.lookup_tag('mytag'),
+                          'somerevid')
+        self.assertEqual(to_tree.branch.last_revision(), revid1)
+
 
 class RedirectingMemoryTransport(memory.MemoryTransport):
 
@@ -597,7 +683,7 @@ class RedirectingMemoryServer(memory.MemoryServer):
 class TestPushRedirect(tests.TestCaseWithTransport):
 
     def setUp(self):
-        tests.TestCaseWithTransport.setUp(self)
+        super(TestPushRedirect, self).setUp()
         self.memory_server = RedirectingMemoryServer()
         self.start_server(self.memory_server)
         # Make the branch and tree that we'll be pushing.
@@ -643,10 +729,8 @@ class TestPushStrictMixin(object):
         self.tree.commit('modify file', rev_id='modified')
 
     def set_config_push_strict(self, value):
-        # set config var (any of bazaar.conf, locations.conf, branch.conf
-        # should do)
-        conf = self.tree.branch.get_config()
-        conf.set_user_option('push_strict', value)
+        br = branch.Branch.open('local')
+        br.get_config_stack().set('push_strict', value)
 
     _default_command = ['push', '../to']
     _default_wd = 'local'
@@ -808,6 +892,55 @@ class TestPushForeign(tests.TestCaseWithTransport):
         target_branch = self.make_dummy_builder('dp').get_branch()
         source_tree = self.make_branch_and_tree("dc")
         output, error = self.run_bzr("push -d dc dp", retcode=3)
-        self.assertEquals("", output)
-        self.assertEquals(error, "bzr: ERROR: It is not possible to losslessly"
+        self.assertEqual("", output)
+        self.assertEqual(error, "bzr: ERROR: It is not possible to losslessly"
             " push to dummy. You may want to use dpush instead.\n")
+
+
+class TestPushOutput(script.TestCaseWithTransportAndScript):
+
+    def test_push_log_format(self):
+        self.run_script("""
+            $ bzr init trunk
+            Created a standalone tree (format: 2a)
+            $ cd trunk
+            $ echo foo > file
+            $ bzr add
+            adding file
+            $ bzr commit -m 'we need some foo'
+            2>Committing to:...trunk/
+            2>added file
+            2>Committed revision 1.
+            $ bzr init ../feature
+            Created a standalone tree (format: 2a)
+            $ bzr push -v ../feature -Olog_format=line
+            Added Revisions:
+            1: jrandom@example.com ...we need some foo
+            2>All changes applied successfully.
+            2>Pushed up to revision 1.
+            """)
+
+    def test_push_with_revspec(self):
+        self.run_script("""
+            $ bzr init-repo .
+            Shared repository with trees (format: 2a)
+            Location:
+              shared repository: .
+            $ bzr init trunk
+            Created a repository tree (format: 2a)
+            Using shared repository...
+            $ cd trunk
+            $ bzr commit -m 'first rev' --unchanged
+            2>Committing to:...trunk/
+            2>Committed revision 1.
+            $ echo foo > file
+            $ bzr add
+            adding file
+            $ bzr commit -m 'we need some foo'
+            2>Committing to:...trunk/
+            2>added file
+            2>Committed revision 2.
+            $ bzr push -r 1 ../other
+            2>Created new branch.
+            $ bzr st ../other # checking that file is not created (#484516)
+            """)
