@@ -18,32 +18,32 @@
 """Tests of the bzr serve command."""
 
 import os
-import os.path
 import signal
-import subprocess
 import sys
 import thread
 import threading
 
 from bzrlib import (
     builtins,
-    debug,
+    config,
     errors,
     osutils,
     revision as _mod_revision,
+    trace,
     transport,
     urlutils,
     )
 from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir
+from bzrlib.controldir import ControlDir
 from bzrlib.smart import client, medium
-from bzrlib.smart.server import BzrServerFactory, SmartTCPServer
+from bzrlib.smart.server import (
+    BzrServerFactory,
+    SmartTCPServer,
+    )
 from bzrlib.tests import (
     TestCaseWithMemoryTransport,
     TestCaseWithTransport,
-    TestSkipped,
     )
-from bzrlib.trace import mutter
 from bzrlib.transport import remote
 
 
@@ -53,37 +53,49 @@ class TestBzrServeBase(TestCaseWithTransport):
                                 *func_args, **func_kwargs):
         """Run 'bzr serve', and run the given func in a thread once the server
         has started.
-        
+
         When 'func' terminates, the server will be terminated too.
-        
+
         Returns stdout and stderr.
         """
-        # install hook
-        def on_server_start(backing_urls, tcp_server):
-            t = threading.Thread(
-                target=on_server_start_thread, args=(tcp_server,))
-            t.start()
         def on_server_start_thread(tcp_server):
+            """This runs concurrently with the server thread.
+
+            The server is interrupted as soon as ``func`` finishes, even if an
+            exception is encountered.
+            """
             try:
                 # Run func if set
                 self.tcp_server = tcp_server
-                if not func is None:
+                if func is not None:
                     try:
                         func(*func_args, **func_kwargs)
                     except Exception, e:
                         # Log errors to make some test failures a little less
                         # mysterious.
-                        mutter('func broke: %r', e)
+                        trace.mutter('func broke: %r', e)
             finally:
                 # Then stop the server
-                mutter('interrupting...')
+                trace.mutter('interrupting...')
                 thread.interrupt_main()
+        # When the hook is fired, it just starts ``on_server_start_thread`` and
+        # return
+        def on_server_start(backing_urls, tcp_server):
+            t = threading.Thread(
+                target=on_server_start_thread, args=(tcp_server,))
+            t.start()
+        # install hook
         SmartTCPServer.hooks.install_named_hook(
             'server_started_ex', on_server_start,
             'run_bzr_serve_then_func hook')
+        # It seesm thread.interrupt_main() will not raise KeyboardInterrupt
+        # until after socket.accept returns. So we set the timeout low to make
+        # the test faster.
+        self.overrideAttr(SmartTCPServer, '_ACCEPT_TIMEOUT', 0.1)
         # start a TCP server
         try:
-            out, err = self.run_bzr(['serve'] + list(serve_args))
+            out, err = self.run_bzr(['serve'] + list(serve_args),
+                                    retcode=retcode)
         except KeyboardInterrupt, e:
             out, err = e.args
         return out, err
@@ -95,6 +107,30 @@ class TestBzrServe(TestBzrServeBase):
         super(TestBzrServe, self).setUp()
         self.disable_missing_extensions_warning()
 
+    def test_server_exception_with_hook(self):
+        """Catch exception from the server in the server_exception hook.
+
+        We use ``run_bzr_serve_then_func`` without a ``func`` so the server
+        will receive a KeyboardInterrupt exception we want to catch.
+        """
+        def hook(exception):
+            if exception[0] is KeyboardInterrupt:
+                sys.stderr.write('catching KeyboardInterrupt\n')
+                return True
+            else:
+                return False
+        SmartTCPServer.hooks.install_named_hook(
+            'server_exception', hook,
+            'test_server_except_hook hook')
+        args = ['--listen', 'localhost', '--port', '0', '--quiet']
+        out, err = self.run_bzr_serve_then_func(args, retcode=0)
+        self.assertEqual('catching KeyboardInterrupt\n', err)
+
+    def test_server_exception_no_hook(self):
+        """test exception without hook returns error"""
+        args = []
+        out, err = self.run_bzr_serve_then_func(args, retcode=3)
+
     def assertInetServerShutsdownCleanly(self, process):
         """Shutdown the server process looking for errors."""
         # Shutdown the server: the server should shut down when it cannot read
@@ -154,7 +190,7 @@ class TestBzrServe(TestBzrServeBase):
             finish_bzr_subprocess, and the base url for the server.
         """
         # Serve from the current directory
-        args = ['serve', '--port', 'localhost:0']
+        args = ['serve', '--listen', 'localhost', '--port', '0']
         args.extend(extra_options)
         process = self.start_bzr_subprocess(args, skip_if_plan_to_signal=True)
         port_line = process.stderr.readline()
@@ -164,10 +200,10 @@ class TestBzrServe(TestBzrServeBase):
         url = 'bzr://localhost:%d/' % port
         self.permit_url(url)
         return process, url
-    
+
     def test_bzr_serve_quiet(self):
         self.make_branch('.')
-        args = ['--port', 'localhost:0', '--quiet']
+        args = ['--listen', 'localhost', '--port', '0', '--quiet']
         out, err = self.run_bzr_serve_then_func(args, retcode=3)
         self.assertEqual('', out)
         self.assertEqual('', err)
@@ -185,7 +221,7 @@ class TestBzrServe(TestBzrServeBase):
         process, transport = self.start_server_inet(['--allow-writes'])
 
         # We get a working branch, and can create a directory
-        branch = BzrDir.open_from_transport(transport).open_branch()
+        branch = ControlDir.open_from_transport(transport).open_branch()
         self.make_read_requests(branch)
         transport.mkdir('adir')
         self.assertInetServerShutsdownCleanly(process)
@@ -193,7 +229,7 @@ class TestBzrServe(TestBzrServeBase):
     def test_bzr_serve_port_readonly(self):
         """bzr server should provide a read only filesystem by default."""
         process, url = self.start_server_port()
-        t = transport.get_transport(url)
+        t = transport.get_transport_from_url(url)
         self.assertRaises(errors.TransportNotPossible, t.mkdir, 'adir')
         self.assertServerFinishesCleanly(process)
 
@@ -227,7 +263,7 @@ class TestBzrServe(TestBzrServeBase):
         log_fname = os.getcwd() + '/server.log'
         self.overrideEnv('BZR_LOG', log_fname)
         process, transport = self.start_server_inet(['-Dhpss'])
-        branch = BzrDir.open_from_transport(transport).open_branch()
+        branch = ControlDir.open_from_transport(transport).open_branch()
         self.make_read_requests(branch)
         self.assertInetServerShutsdownCleanly(process)
         f = open(log_fname, 'rb')
@@ -235,6 +271,75 @@ class TestBzrServe(TestBzrServeBase):
         f.close()
         self.assertContainsRe(content, r'hpss request: \[[0-9-]+\]')
 
+    def test_bzr_serve_supports_configurable_timeout(self):
+        gs = config.GlobalStack()
+        gs.set('serve.client_timeout', 0.2)
+        # Save the config as the subprocess will use it
+        gs.store.save()
+        process, url = self.start_server_port()
+        self.build_tree_contents([('a_file', 'contents\n')])
+        # We can connect and issue a request
+        t = transport.get_transport_from_url(url)
+        self.assertEqual('contents\n', t.get_bytes('a_file'))
+        # However, if we just wait for more content from the server, it will
+        # eventually disconnect us.
+        m = t.get_smart_medium()
+        m.read_bytes(1)
+        # Now, we wait for timeout to trigger
+        err = process.stderr.readline()
+        self.assertEqual(
+            'Connection Timeout: disconnecting client after 0.2 seconds\n',
+            err)
+        self.assertServerFinishesCleanly(process)
+
+    def test_bzr_serve_supports_client_timeout(self):
+        process, url = self.start_server_port(['--client-timeout=0.1'])
+        self.build_tree_contents([('a_file', 'contents\n')])
+        # We can connect and issue a request
+        t = transport.get_transport_from_url(url)
+        self.assertEqual('contents\n', t.get_bytes('a_file'))
+        # However, if we just wait for more content from the server, it will
+        # eventually disconnect us.
+        # TODO: Use something like signal.alarm() so that if the server doesn't
+        #       properly handle the timeout, we end up failing the test instead
+        #       of hanging forever.
+        m = t.get_smart_medium()
+        m.read_bytes(1)
+        # Now, we wait for timeout to trigger
+        err = process.stderr.readline()
+        self.assertEqual(
+            'Connection Timeout: disconnecting client after 0.1 seconds\n',
+            err)
+        self.assertServerFinishesCleanly(process)
+
+    def test_bzr_serve_graceful_shutdown(self):
+        big_contents = 'a'*64*1024
+        self.build_tree_contents([('bigfile', big_contents)])
+        process, url = self.start_server_port(['--client-timeout=1.0'])
+        t = transport.get_transport_from_url(url)
+        m = t.get_smart_medium()
+        c = client._SmartClient(m)
+        # Start, but don't finish a response
+        resp, response_handler = c.call_expecting_body('get', 'bigfile')
+        self.assertEqual(('ok',), resp)
+        # Note: process.send_signal is a Python 2.6ism
+        process.send_signal(signal.SIGHUP)
+        # Wait for the server to notice the signal, and then read the actual
+        # body of the response. That way we know that it is waiting for the
+        # request to finish
+        self.assertEqual('Requested to stop gracefully\n',
+                         process.stderr.readline())
+        self.assertEqual('Waiting for 1 client(s) to finish\n',
+                         process.stderr.readline())
+        body = response_handler.read_body_bytes()
+        if body != big_contents:
+            self.fail('Failed to properly read the contents of "bigfile"')
+        # Now that our request is finished, the medium should notice it has
+        # been disconnected.
+        self.assertEqual('', m.read_bytes(1))
+        # And the server should be stopping
+        self.assertEqual(0, process.wait())
+
 
 class TestCmdServeChrooting(TestBzrServeBase):
 
@@ -248,7 +353,7 @@ class TestCmdServeChrooting(TestBzrServeBase):
         t = self.get_transport()
         t.mkdir('server-root')
         self.run_bzr_serve_then_func(
-            ['--port', '127.0.0.1:0',
+            ['--listen', '127.0.0.1', '--port', '0',
              '--directory', t.local_abspath('server-root'),
              '--allow-writes'],
             func=self.when_server_started)
@@ -276,7 +381,8 @@ class TestCmdServeChrooting(TestBzrServeBase):
 
 class TestUserdirExpansion(TestCaseWithMemoryTransport):
 
-    def fake_expanduser(self, path):
+    @staticmethod
+    def fake_expanduser(path):
         """A simple, environment-independent, function for the duration of this
         test.
 
@@ -295,7 +401,7 @@ class TestUserdirExpansion(TestCaseWithMemoryTransport):
             self.fake_expanduser, lambda t: base_path)
         mem_transport = self.get_transport()
         mem_transport.mkdir_multi(['home', 'home/user'])
-        bzr_server.set_up(mem_transport, None, None, inet=True)
+        bzr_server.set_up(mem_transport, None, None, inet=True, timeout=4.0)
         self.addCleanup(bzr_server.tear_down)
         return bzr_server
 
@@ -317,7 +423,7 @@ class TestUserdirExpansion(TestCaseWithMemoryTransport):
         base_url = urlutils.local_path_to_url(base_dir) + '/'
         # Define a fake 'protocol' to capture the transport that cmd_serve
         # passes to serve_bzr.
-        def capture_transport(transport, host, port, inet):
+        def capture_transport(transport, host, port, inet, timeout):
             self.bzr_serve_transport = transport
         cmd = builtins.cmd_serve()
         # Read-only
@@ -334,4 +440,10 @@ class TestUserdirExpansion(TestCaseWithMemoryTransport):
         self.assertEqual(base_url, self.bzr_serve_transport.base)
         self.assertEqual(base_dir,
             server_maker.get_base_path(self.bzr_serve_transport))
-
+        # Read-only, from a URL
+        cmd.run(directory=base_url, protocol=capture_transport)
+        server_maker = BzrServerFactory()
+        self.assertEqual(
+            'readonly+%s' % base_url, self.bzr_serve_transport.base)
+        self.assertEqual(
+            base_dir, server_maker.get_base_path(self.bzr_serve_transport))
