@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010, 2012, 2016 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 
 import os
 
-from bzrlib.bzrdir import BzrDir
+from bzrlib.controldir import ControlDir
 from bzrlib.filters import ContentFilter
 from bzrlib.switch import switch
 from bzrlib.workingtree import WorkingTree
@@ -123,16 +123,12 @@ class TestWorkingTreeWithContentFilters(TestCaseWithWorkingTree):
         # the easiest way currently given we don't have easy access to the
         # WorkingTree after it is created but before the filter stack is used
         # to populate content.
-        self.real_content_filter_stack = WorkingTree._content_filter_stack
-        def restore_real_content_filter_stack():
-            WorkingTree._content_filter_stack = self.real_content_filter_stack
-        self.addCleanup(restore_real_content_filter_stack)
-        def _content_filter_stack(tree, path=None, file_id=None):
+        def new_stack(tree, path=None, file_id=None):
             if path.endswith('.txt'):
                 return [ContentFilter(_swapcase, _swapcase)]
             else:
                 return []
-        WorkingTree._content_filter_stack = _content_filter_stack
+        self.overrideAttr(WorkingTree, '_content_filter_stack', new_stack)
 
     def assert_basis_content(self, expected_content, branch, file_id):
         # Note: We need to use try/finally here instead of addCleanup()
@@ -241,15 +237,15 @@ class TestWorkingTreeWithContentFilters(TestCaseWithWorkingTree):
         self.addCleanup(source.unlock)
 
         expected_canonical_form = 'Foo Txt\nend string\n'
-        self.assertEquals(source.get_file(txt_fileid, filtered=True).read(),
+        self.assertEqual(source.get_file(txt_fileid, filtered=True).read(),
             expected_canonical_form)
-        self.assertEquals(source.get_file(txt_fileid, filtered=False).read(),
+        self.assertEqual(source.get_file(txt_fileid, filtered=False).read(),
             'Foo Txt')
 
         # results are: kind, size, executable, sha1_or_link_target
         result = source.path_content_summary('file1.txt')
 
-        self.assertEquals(result,
+        self.assertEqual(result,
             ('file', None, False, None))
 
         # we could give back the length of the canonical form, but in general
@@ -334,7 +330,7 @@ class TestWorkingTreeWithContentFilters(TestCaseWithWorkingTree):
         self.assertFileEqual("fOO tXT", 'checkout/file1.txt')
 
         # Switch it to branch-b and check the tree is updated
-        checkout_control_dir = BzrDir.open_containing('checkout')[0]
+        checkout_control_dir = ControlDir.open_containing('checkout')[0]
         switch(checkout_control_dir, source.branch)
         self.assertFileEqual("fOO rocks!", 'checkout/file1.txt')
         self.assertFileEqual("hELLO wORLD", 'checkout/file4.txt')
