@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010, 2016 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,6 +16,8 @@
 
 """Tests for bencode structured encoding"""
 
+import sys
+
 from bzrlib import tests
 
 def load_tests(standard_tests, module, loader):
@@ -29,7 +31,7 @@ class TestBencodeDecode(tests.TestCase):
     module = None
 
     def _check(self, expected, source):
-        self.assertEquals(expected, self.module.bdecode(source))
+        self.assertEqual(expected, self.module.bdecode(source))
 
     def _run_check_error(self, exc, bad):
         """Check that bdecoding a string raises a particular exception."""
@@ -105,7 +107,14 @@ class TestBencodeDecode(tests.TestCase):
                             'd8:spam.mp3d6:author5:Alice6:lengthi100000eee')
 
     def test_dict_deepnested(self):
-        self._run_check_error(RuntimeError, ("d0:" * 10000) + 'i1e' + ("e" * 10000))
+        # The recursion here provokes CPython into emitting a warning on
+        # stderr, "maximum recursion depth exceeded in __subclasscheck__", due
+        # to running out of stack space while evaluating "except (...):" in
+        # _bencode_py.  This is harmless, so we temporarily override stderr to
+        # avoid distracting noise in the test output.
+        self.overrideAttr(sys, 'stderr', self._log_file)
+        self._run_check_error(
+            RuntimeError, ("d0:" * 10000) + 'i1e' + ("e" * 10000))
 
     def test_malformed_dict(self):
         self._run_check_error(ValueError, 'd')
@@ -146,7 +155,7 @@ class TestBencodeEncode(tests.TestCase):
     module = None
 
     def _check(self, expected, source):
-        self.assertEquals(expected, self.module.bencode(source))
+        self.assertEqual(expected, self.module.bencode(source))
 
     def test_int(self):
         self._check('i4e', 4)
