@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2009, 2011 Canonical Ltd
+# Copyright (C) 2005-2011, 2016 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,13 +22,14 @@ import gzip
 
 import bzrlib.errors as errors
 from bzrlib.errors import BzrError
+from bzrlib.store import TransportStore
 from bzrlib.store.text import TextStore
+from bzrlib.store.versioned import VersionedFileStore
 from bzrlib.tests import TestCase, TestCaseInTempDir, TestCaseWithTransport
-import bzrlib.store as store
-import bzrlib.store.versioned
 import bzrlib.transactions as transactions
 import bzrlib.transport as transport
 from bzrlib.transport.memory import MemoryTransport
+from bzrlib.weave import WeaveFile
 
 
 class TestStores(object):
@@ -81,7 +82,7 @@ class TestStores(object):
 class TestCompressedTextStore(TestCaseInTempDir, TestStores):
 
     def get_store(self, path=u'.'):
-        t = transport.get_transport(path)
+        t = transport.get_transport_from_path(path)
         return TextStore(t, compressed=True)
 
     def test_total_size(self):
@@ -138,7 +139,7 @@ class TestMemoryStore(TestCase):
 class TestTextStore(TestCaseInTempDir, TestStores):
 
     def get_store(self, path=u'.'):
-        t = transport.get_transport(path)
+        t = transport.get_transport_from_path(path)
         return TextStore(t, compressed=False)
 
     def test_total_size(self):
@@ -156,7 +157,7 @@ class TestTextStore(TestCaseInTempDir, TestStores):
 class TestMixedTextStore(TestCaseInTempDir, TestStores):
 
     def get_store(self, path=u'.', compressed=True):
-        t = transport.get_transport(path)
+        t = transport.get_transport_from_path(path)
         return TextStore(t, compressed=compressed)
 
     def test_get_mixed(self):
@@ -167,24 +168,24 @@ class TestMixedTextStore(TestCaseInTempDir, TestStores):
         self.assertPathExists('a.gz')
         self.assertFalse(os.path.lexists('a'))
 
-        self.assertEquals(gzip.GzipFile('a.gz').read(), 'hello there')
+        self.assertEqual(gzip.GzipFile('a.gz').read(), 'hello there')
 
-        self.assertEquals(cs.has_id('a'), True)
-        self.assertEquals(s.has_id('a'), True)
-        self.assertEquals(cs.get('a').read(), 'hello there')
-        self.assertEquals(s.get('a').read(), 'hello there')
+        self.assertEqual(cs.has_id('a'), True)
+        self.assertEqual(s.has_id('a'), True)
+        self.assertEqual(cs.get('a').read(), 'hello there')
+        self.assertEqual(s.get('a').read(), 'hello there')
 
         self.assertRaises(BzrError, s.add, StringIO('goodbye'), 'a')
 
         s.add(StringIO('goodbye'), 'b')
         self.assertPathExists('b')
         self.assertFalse(os.path.lexists('b.gz'))
-        self.assertEquals(open('b').read(), 'goodbye')
+        self.assertEqual(open('b').read(), 'goodbye')
 
-        self.assertEquals(cs.has_id('b'), True)
-        self.assertEquals(s.has_id('b'), True)
-        self.assertEquals(cs.get('b').read(), 'goodbye')
-        self.assertEquals(s.get('b').read(), 'goodbye')
+        self.assertEqual(cs.has_id('b'), True)
+        self.assertEqual(s.has_id('b'), True)
+        self.assertEqual(cs.get('b').read(), 'goodbye')
+        self.assertEqual(s.get('b').read(), 'goodbye')
 
         self.assertRaises(BzrError, cs.add, StringIO('again'), 'b')
 
@@ -203,7 +204,7 @@ class MockTransport(transport.Transport):
         return
 
 
-class InstrumentedTransportStore(store.TransportStore):
+class InstrumentedTransportStore(TransportStore):
     """An instrumented TransportStore.
 
     Here we replace template method worker methods with calls that record the
@@ -241,37 +242,37 @@ class TestMockTransport(TestCase):
 class TestTransportStore(TestCase):
 
     def test__relpath_invalid(self):
-        my_store = store.TransportStore(MockTransport())
+        my_store = TransportStore(MockTransport())
         self.assertRaises(ValueError, my_store._relpath, '/foo')
         self.assertRaises(ValueError, my_store._relpath, 'foo/')
 
     def test_register_invalid_suffixes(self):
-        my_store = store.TransportStore(MockTransport())
+        my_store = TransportStore(MockTransport())
         self.assertRaises(ValueError, my_store.register_suffix, '/')
         self.assertRaises(ValueError, my_store.register_suffix, '.gz/bar')
 
     def test__relpath_unregister_suffixes(self):
-        my_store = store.TransportStore(MockTransport())
+        my_store = TransportStore(MockTransport())
         self.assertRaises(ValueError, my_store._relpath, 'foo', ['gz'])
         self.assertRaises(ValueError, my_store._relpath, 'foo', ['dsc', 'gz'])
 
     def test__relpath_simple(self):
-        my_store = store.TransportStore(MockTransport())
+        my_store = TransportStore(MockTransport())
         self.assertEqual("foo", my_store._relpath('foo'))
 
     def test__relpath_prefixed(self):
-        my_store = store.TransportStore(MockTransport(), True)
+        my_store = TransportStore(MockTransport(), True)
         self.assertEqual('45/foo', my_store._relpath('foo'))
 
     def test__relpath_simple_suffixed(self):
-        my_store = store.TransportStore(MockTransport())
+        my_store = TransportStore(MockTransport())
         my_store.register_suffix('bar')
         my_store.register_suffix('baz')
         self.assertEqual('foo.baz', my_store._relpath('foo', ['baz']))
         self.assertEqual('foo.bar.baz', my_store._relpath('foo', ['bar', 'baz']))
 
     def test__relpath_prefixed_suffixed(self):
-        my_store = store.TransportStore(MockTransport(), True)
+        my_store = TransportStore(MockTransport(), True)
         my_store.register_suffix('bar')
         my_store.register_suffix('baz')
         self.assertEqual('45/foo.baz', my_store._relpath('foo', ['baz']))
@@ -393,15 +394,15 @@ class TestTransportStore(TestCase):
         self.assertRaises(KeyError, to_store.get, 'missing', 'sig')
 
     def test_relpath_escaped(self):
-        my_store = store.TransportStore(MemoryTransport())
+        my_store = TransportStore(MemoryTransport())
         self.assertEqual('%25', my_store._relpath('%'))
 
     def test_escaped_uppercase(self):
         """Uppercase letters are escaped for safety on Windows"""
-        my_store = store.TransportStore(MemoryTransport(), prefixed=True,
+        my_store = TransportStore(MemoryTransport(), prefixed=True,
             escaped=True)
         # a particularly perverse file-id! :-)
-        self.assertEquals(my_store._relpath('C:<>'), 'be/%2543%253a%253c%253e')
+        self.assertEqual(my_store._relpath('C:<>'), 'be/%2543%253a%253c%253e')
 
 
 class TestVersionFileStore(TestCaseWithTransport):
@@ -411,7 +412,8 @@ class TestVersionFileStore(TestCaseWithTransport):
 
     def setUp(self):
         super(TestVersionFileStore, self).setUp()
-        self.vfstore = store.versioned.VersionedFileStore(MemoryTransport())
+        self.vfstore = VersionedFileStore(MemoryTransport(),
+            versionedfile_class=WeaveFile)
         self.vfstore.get_scope = self.get_scope
         self._transaction = None
 
@@ -436,8 +438,8 @@ class TestVersionFileStore(TestCaseWithTransport):
         self.assertRaises(errors.ReadOnlyError, vf.add_lines, 'b', [], [])
 
     def test___iter__escaped(self):
-        self.vfstore = store.versioned.VersionedFileStore(MemoryTransport(),
-            prefixed=True, escaped=True)
+        self.vfstore = VersionedFileStore(MemoryTransport(),
+            prefixed=True, escaped=True, versionedfile_class=WeaveFile)
         self.vfstore.get_scope = self.get_scope
         self._transaction = transactions.WriteTransaction()
         vf = self.vfstore.get_weave_or_empty(' ', self._transaction)
