@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2011 Canonical Ltd
+# Copyright (C) 2005-2012, 2016 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -56,13 +56,12 @@ class TestUIConfiguration(tests.TestCaseWithTransport):
 
     def test_output_encoding_configuration(self):
         enc = fixtures.generate_unicode_encodings().next()
-        config.GlobalConfig().set_user_option('output_encoding',
-            enc)
+        config.GlobalStack().set('output_encoding', enc)
         ui = tests.TestUIFactory(stdin=None,
             stdout=tests.StringIOWrapper(),
             stderr=tests.StringIOWrapper())
         output = ui.make_output_stream()
-        self.assertEquals(output.encoding, enc)
+        self.assertEqual(output.encoding, enc)
 
 
 class TestTextUIFactory(tests.TestCase):
@@ -76,7 +75,7 @@ class TestTextUIFactory(tests.TestCase):
     def test_text_factory_confirm(self):
         # turns into reading a regular boolean
         ui = self.make_test_ui_factory('n\n')
-        self.assertEquals(ui.confirm_action(u'Should %(thing)s pass?',
+        self.assertEqual(ui.confirm_action(u'Should %(thing)s pass?',
             'bzrlib.tests.test_ui.confirmation',
             {'thing': 'this'},),
             False)
@@ -98,28 +97,19 @@ class TestTextUIFactory(tests.TestCase):
             pb.finished()
 
     def test_text_factory_utf8_password(self):
-        """Test an utf8 password.
-
-        We can't predict what encoding users will have for stdin, so we force
-        it to utf8 to test that we transport the password correctly.
-        """
-        ui = self.make_test_ui_factory(u'baz\u1234'.encode('utf8'))
+        """Test an utf8 password."""
+        ui = _mod_ui_text.TextUIFactory(None, None, None)
+        ui.stdin = tests.StringIOWrapper(u'baz\u1234'.encode('utf8'))
+        ui.stdout = tests.StringIOWrapper()
+        ui.stderr = tests.StringIOWrapper()
         ui.stderr.encoding = ui.stdout.encoding = ui.stdin.encoding = 'utf8'
-        pb = ui.nested_progress_bar()
-        try:
-            password = self.apply_redirected(ui.stdin, ui.stdout, ui.stderr,
-                                             ui.get_password,
-                                             u'Hello \u1234 %(user)s',
-                                             user=u'some\u1234')
-            # We use StringIO objects, we need to decode them
-            self.assertEqual(u'baz\u1234', password.decode('utf8'))
-            self.assertEqual(u'Hello \u1234 some\u1234: ',
-                             ui.stderr.getvalue().decode('utf8'))
-            # stdin and stdout should be empty
-            self.assertEqual('', ui.stdin.readline())
-            self.assertEqual('', ui.stdout.readline())
-        finally:
-            pb.finished()
+        password = ui.get_password(u'Hello \u1234 %(user)s', user=u'some\u1234')
+        self.assertEqual(u'baz\u1234', password)
+        self.assertEqual(u'Hello \u1234 some\u1234: ',
+                         ui.stderr.getvalue().decode('utf8'))
+        # stdin and stdout should be empty
+        self.assertEqual('', ui.stdin.readline())
+        self.assertEqual('', ui.stdout.getvalue())
 
     def test_text_ui_get_boolean(self):
         stdin = tests.StringIOWrapper("y\n" # True
@@ -271,36 +261,30 @@ class TestTextUIFactory(tests.TestCase):
             pb.finished()
 
     def test_text_ui_getusername(self):
-        factory = _mod_ui_text.TextUIFactory(None, None, None)
-        factory.stdin = tests.StringIOWrapper("someuser\n\n")
-        factory.stdout = tests.StringIOWrapper()
-        factory.stderr = tests.StringIOWrapper()
-        factory.stdout.encoding = "utf8"
-        # there is no output from the base factory
-        self.assertEqual("someuser",
-                         factory.get_username(u'Hello %(host)s', host='some'))
-        self.assertEquals("Hello some: ", factory.stderr.getvalue())
-        self.assertEquals('', factory.stdout.getvalue())
-        self.assertEqual("", factory.get_username(u"Gebruiker"))
+        ui = _mod_ui_text.TextUIFactory(None, None, None)
+        ui.stdin = tests.StringIOWrapper('someuser\n\n')
+        ui.stdout = tests.StringIOWrapper()
+        ui.stderr = tests.StringIOWrapper()
+        ui.stdout.encoding = 'utf8'
+        self.assertEqual('someuser',
+                         ui.get_username(u'Hello %(host)s', host='some'))
+        self.assertEqual('Hello some: ', ui.stderr.getvalue())
+        self.assertEqual('', ui.stdout.getvalue())
+        self.assertEqual('', ui.get_username(u"Gebruiker"))
         # stdin should be empty
-        self.assertEqual('', factory.stdin.readline())
+        self.assertEqual('', ui.stdin.readline())
 
     def test_text_ui_getusername_utf8(self):
-        ui = tests.TestUIFactory(stdin=u'someuser\u1234'.encode('utf8'),
-                                 stdout=tests.StringIOWrapper(),
-                                 stderr=tests.StringIOWrapper())
+        ui = _mod_ui_text.TextUIFactory(None, None, None)
+        ui.stdin = tests.StringIOWrapper(u'someuser\u1234'.encode('utf8'))
+        ui.stdout = tests.StringIOWrapper()
+        ui.stderr = tests.StringIOWrapper()
         ui.stderr.encoding = ui.stdout.encoding = ui.stdin.encoding = "utf8"
-        pb = ui.nested_progress_bar()
-        try:
-            # there is no output from the base factory
-            username = self.apply_redirected(ui.stdin, ui.stdout, ui.stderr,
-                ui.get_username, u'Hello\u1234 %(host)s', host=u'some\u1234')
-            self.assertEquals(u"someuser\u1234", username.decode('utf8'))
-            self.assertEquals(u"Hello\u1234 some\u1234: ",
-                              ui.stderr.getvalue().decode("utf8"))
-            self.assertEquals('', ui.stdout.getvalue())
-        finally:
-            pb.finished()
+        username = ui.get_username(u'Hello %(host)s', host=u'some\u1234')
+        self.assertEqual(u"someuser\u1234", username)
+        self.assertEqual(u"Hello some\u1234: ",
+                          ui.stderr.getvalue().decode("utf8"))
+        self.assertEqual('', ui.stdout.getvalue())
 
     def test_quietness(self):
         self.overrideEnv('BZR_PROGRESS_BAR', 'text')
@@ -323,8 +307,8 @@ class TestTextUIFactory(tests.TestCase):
         remote_fmt._network_name = RepositoryFormatKnitPack5().network_name()
         ui.show_user_warning('cross_format_fetch', from_format=RepositoryFormat2a(),
             to_format=remote_fmt)
-        self.assertEquals('', out.getvalue())
-        self.assertEquals("Doing on-the-fly conversion from RepositoryFormat2a() to "
+        self.assertEqual('', out.getvalue())
+        self.assertEqual("Doing on-the-fly conversion from RepositoryFormat2a() to "
             "RemoteRepositoryFormat(_network_name='Bazaar RepositoryFormatKnitPack5 "
             "(bzr 1.6)\\n').\nThis may take some time. Upgrade the repositories to "
             "the same format for better performance.\n",
@@ -336,8 +320,8 @@ class TestTextUIFactory(tests.TestCase):
         ui.suppressed_warnings.add('cross_format_fetch')
         ui.show_user_warning('cross_format_fetch', from_format=RepositoryFormat2a(),
             to_format=remote_fmt)
-        self.assertEquals('', out.getvalue())
-        self.assertEquals('', err.getvalue())
+        self.assertEqual('', out.getvalue())
+        self.assertEqual('', err.getvalue())
 
 
 class TestTextUIOutputStream(tests.TestCase):
@@ -465,11 +449,11 @@ class TestBoolFromString(tests.TestCase):
 
     def assertIsTrue(self, s, accepted_values=None):
         res = _mod_ui.bool_from_string(s, accepted_values=accepted_values)
-        self.assertEquals(True, res)
+        self.assertEqual(True, res)
 
     def assertIsFalse(self, s, accepted_values=None):
         res = _mod_ui.bool_from_string(s, accepted_values=accepted_values)
-        self.assertEquals(False, res)
+        self.assertEqual(False, res)
 
     def assertIsNone(self, s, accepted_values=None):
         res = _mod_ui.bool_from_string(s, accepted_values=accepted_values)
@@ -522,7 +506,7 @@ class TestConfirmationUserInterfacePolicy(tests.TestCase):
     def test_confirm_action_default(self):
         base_ui = _mod_ui.NoninteractiveUIFactory()
         for answer in [True, False]:
-            self.assertEquals(
+            self.assertEqual(
                 _mod_ui.ConfirmationUserInterfacePolicy(base_ui, answer, {})
                 .confirm_action("Do something?",
                     "bzrlib.tests.do_something", {}),
@@ -537,9 +521,9 @@ class TestConfirmationUserInterfacePolicy(tests.TestCase):
                         base_ui, default_answer, dict(given_id=specific_answer))
                     result = wrapper.confirm_action("Do something?", conf_id, {})
                     if conf_id == 'given_id':
-                        self.assertEquals(result, specific_answer)
+                        self.assertEqual(result, specific_answer)
                     else:
-                        self.assertEquals(result, default_answer)
+                        self.assertEqual(result, default_answer)
 
     def test_repr(self):
         base_ui = _mod_ui.NoninteractiveUIFactory()
