@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2011 Canonical Ltd
+# Copyright (C) 2007-2012, 2016 Canonical Ltd
 # Authors:  Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -88,11 +88,11 @@ class TestWorkingTreeFormat4(TestCaseWithTransport):
         self.assertIsDirectory('.bzr/checkout', t)
         self.assertIsDirectory('.bzr/checkout/lock', t)
         our_lock = LockDir(t, '.bzr/checkout/lock')
-        self.assertEquals(our_lock.peek(), None)
+        self.assertEqual(our_lock.peek(), None)
         tree.lock_write()
         self.assertTrue(our_lock.peek())
         tree.unlock()
-        self.assertEquals(our_lock.peek(), None)
+        self.assertEqual(our_lock.peek(), None)
 
     def make_workingtree(self, relpath=''):
         url = self.get_url(relpath)
@@ -212,12 +212,12 @@ class TestWorkingTreeFormat4(TestCaseWithTransport):
         rev1 = subtree.commit('commit in subdir')
         rev1_tree = subtree.basis_tree()
         rev1_tree.lock_read()
-        rev1_tree.inventory
+        rev1_tree.root_inventory
         self.addCleanup(rev1_tree.unlock)
         rev2 = subtree.commit('second commit in subdir', allow_pointless=True)
         rev2_tree = subtree.basis_tree()
         rev2_tree.lock_read()
-        rev2_tree.inventory
+        rev2_tree.root_inventory
         self.addCleanup(rev2_tree.unlock)
 
         tree.branch.pull(subtree.branch)
@@ -516,7 +516,7 @@ class TestWorkingTreeFormat4(TestCaseWithTransport):
 
     def test_unique_root_id_per_tree(self):
         # each time you initialize a new tree, it gets a different root id
-        format_name = 'dirstate-with-subtree'
+        format_name = 'development-subtree'
         tree1 = self.make_branch_and_tree('tree1',
             format=format_name)
         tree2 = self.make_branch_and_tree('tree2',
@@ -552,12 +552,12 @@ class TestWorkingTreeFormat4(TestCaseWithTransport):
         tree = self.make_branch_and_tree('tag', format='dirstate-tags')
         self.assertEqual(inventory.ROOT_ID, tree.get_root_id())
         tree = self.make_branch_and_tree('subtree',
-                                         format='dirstate-with-subtree')
+                                         format='development-subtree')
         self.assertNotEqual(inventory.ROOT_ID, tree.get_root_id())
 
     def test_non_subtree_with_nested_trees(self):
         # prior to dirstate, st/diff/commit ignored nested trees.
-        # dirstate, as opposed to dirstate-with-subtree, should
+        # dirstate, as opposed to development-subtree, should
         # behave the same way.
         tree = self.make_branch_and_tree('.', format='dirstate')
         self.assertFalse(tree.supports_tree_reference())
@@ -603,8 +603,8 @@ class TestWorkingTreeFormat4(TestCaseWithTransport):
         tree.unlock()
 
     def test_with_subtree_supports_tree_references(self):
-        # dirstate-with-subtree should support tree-references.
-        tree = self.make_branch_and_tree('.', format='dirstate-with-subtree')
+        # development-subtree should support tree-references.
+        tree = self.make_branch_and_tree('.', format='development-subtree')
         self.assertTrue(tree.supports_tree_reference())
         # having checked this is on, the tree interface, and intertree
         # interface tests, will proceed to test the subtree support of
@@ -866,7 +866,7 @@ class TestInventoryCoherency(TestCaseWithTransport):
         self.addCleanup(tree.unlock)
         # Force access to the in memory inventory to trigger bug #494221: try
         # maintaining the in-memory inventory
-        inv = tree.inventory
+        inv = tree.root_inventory
         self.assertTrue(inv.has_id('a-id'))
         self.assertTrue(inv.has_id('b-id'))
         tree.unversion(['a-id', 'b-id'])
