@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2009 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011, 2016 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,11 +16,13 @@
 
 """Implementation of Transport that adapts another transport to be readonly."""
 
+from __future__ import absolute_import
+
 from bzrlib.errors import TransportNotPossible, NoSmartMedium
-from bzrlib.transport.decorator import TransportDecorator, DecoratorServer
+from bzrlib.transport import decorator
 
 
-class ReadonlyTransportDecorator(TransportDecorator):
+class ReadonlyTransportDecorator(decorator.TransportDecorator):
     """A decorator that can convert any transport to be readonly.
 
     This is requested via the 'readonly+' prefix to get_transport().
@@ -39,6 +41,10 @@ class ReadonlyTransportDecorator(TransportDecorator):
         """Readonly transport decorators are invoked via 'readonly+'"""
         return 'readonly+'
 
+    def rename(self, rel_from, rel_to):
+        """See Transport.rename."""
+        raise TransportNotPossible('readonly transport')
+
     def delete(self, relpath):
         """See Transport.delete()."""
         raise TransportNotPossible('readonly transport')
@@ -59,6 +65,10 @@ class ReadonlyTransportDecorator(TransportDecorator):
         """See Transport.mkdir()."""
         raise TransportNotPossible('readonly transport')
 
+    def open_write_stream(self, relpath, mode=None):
+        """See Transport.open_write_stream()."""
+        raise TransportNotPossible('readonly transport')
+
     def is_readonly(self):
         """See Transport.is_readonly."""
         return True
@@ -72,21 +82,13 @@ class ReadonlyTransportDecorator(TransportDecorator):
         raise TransportNotPossible('readonly transport')
 
     def get_smart_client(self):
-        raise NoSmartServer(self.base)
+        raise NoSmartMedium(self)
 
     def get_smart_medium(self):
         raise NoSmartMedium(self)
 
 
-
-class ReadonlyServer(DecoratorServer):
-    """Server for the ReadonlyTransportDecorator for testing with."""
-
-    def get_decorator_class(self):
-        return ReadonlyTransportDecorator
-
-
 def get_test_permutations():
     """Return the permutations to be used in testing."""
-    return [(ReadonlyTransportDecorator, ReadonlyServer),
-            ]
+    from bzrlib.tests import test_server
+    return [(ReadonlyTransportDecorator, test_server.ReadonlyServer)]
