@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,13 +14,20 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib import (
     cache_utf8,
     errors,
     inventory,
     xml6,
-    xml8,
     )
+from bzrlib.xml_serializer import (
+    encode_and_escape,
+    get_utf8_or_ascii,
+    unpack_inventory_entry,
+    )
+
 
 class Serializer_v5(xml6.Serializer_v6):
     """Version 5 serializer
@@ -35,7 +42,7 @@ class Serializer_v5(xml6.Serializer_v6):
         """Construct from XML Element
         """
         root_id = elt.get('file_id') or inventory.ROOT_ID
-        root_id = xml8._get_utf8_or_ascii(root_id)
+        root_id = get_utf8_or_ascii(root_id)
 
         format = elt.get('format')
         if format is not None:
@@ -52,10 +59,9 @@ class Serializer_v5(xml6.Serializer_v6):
         #   avoiding attributes     2.46s
         #   adding assertions       2.50s
         #   last_parent cache       2.52s (worse, removed)
-        unpack_entry = self._unpack_entry
         byid = inv._byid
         for e in elt:
-            ie = unpack_entry(e, entry_cache=entry_cache,
+            ie = unpack_inventory_entry(e, entry_cache=entry_cache,
                               return_from_cache=return_from_cache)
             parent_id = ie.parent_id
             if parent_id is None:
@@ -92,13 +98,13 @@ class Serializer_v5(xml6.Serializer_v6):
         """Append the inventory root to output."""
         if inv.root.file_id not in (None, inventory.ROOT_ID):
             fileid1 = ' file_id="'
-            fileid2 = xml8._encode_and_escape(inv.root.file_id)
+            fileid2 = encode_and_escape(inv.root.file_id)
         else:
             fileid1 = ""
             fileid2 = ""
         if inv.revision_id is not None:
             revid1 = ' revision_id="'
-            revid2 = xml8._encode_and_escape(inv.revision_id)
+            revid2 = encode_and_escape(inv.revision_id)
         else:
             revid1 = ""
             revid2 = ""
